@@ -7,7 +7,6 @@
  */
 
 #include "common/macros.h"
-#include <stdint.h>
 #define _LGPL_SOURCE
 #include <assert.h>
 #include <ctype.h>
@@ -224,44 +223,77 @@ end:
 LTTNG_HIDDEN
 int utils_create_pid_file(pid_t pid, const char *filepath)
 {
-	int ret, fd = -1;
-	FILE *fp = NULL;
+	int ret;
+	FILE *fp;
 
 	assert(filepath);
 
-	fd = open(filepath, O_CREAT | O_WRONLY, S_IRUSR |S_IWUSR | S_IRGRP | S_IROTH);
-	if (fd < 0) {
-		PERROR("open file %s", filepath);
-		ret = -1;
-		goto error;
-	}
-
-	fp = fdopen(fd, "w");
+	fp = fopen(filepath, "w");
 	if (fp == NULL) {
-		PERROR("fdopen file %s", filepath);
+		PERROR("open pid file %s", filepath);
 		ret = -1;
-		if (close(fd)) {
-			PERROR("Failed to close `%s` file descriptor while handling fdopen error", filepath);
-		}
-
 		goto error;
 	}
 
 	ret = fprintf(fp, "%d\n", (int) pid);
 	if (ret < 0) {
-		PERROR("fprintf file %s", filepath);
-		ret = -1;
+		PERROR("fprintf pid file");
 		goto error;
 	}
 
-	DBG("'%d' written in file %s", (int) pid, filepath);
+	if (fclose(fp)) {
+		PERROR("fclose");
+	}
+	DBG("Pid %d written in file %s", (int) pid, filepath);
 	ret = 0;
+error:
+	return ret;
+}
+
+/*
+ * Create lock file to the given path and filename.
+ * Returns the associated file descriptor, -1 on error.
+ */
+LTTNG_HIDDEN
+int utils_create_lock_file(const char *filepath)
+{
+	int ret;
+	int fd;
+	struct flock lock;
+
+	assert(filepath);
+
+	memset(&lock, 0, sizeof(lock));
+	fd = open(filepath, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR |
+		S_IRGRP | S_IWGRP);
+	if (fd < 0) {
+		PERROR("open lock file %s", filepath);
+		fd = -1;
+		goto error;
+	}
+
+	/*
+	 * Attempt to lock the file. If this fails, there is
+	 * already a process using the same lock file running
+	 * and we should exit.
+	 */
+	lock.l_whence = SEEK_SET;
+	lock.l_type = F_WRLCK;
+
+	ret = fcntl(fd, F_SETLK, &lock);
+	if (ret == -1) {
+		PERROR("fcntl lock file");
+		ERR("Could not get lock file %s, another instance is running.",
+			filepath);
+		if (close(fd)) {
+			PERROR("close lock file");
+		}
+		fd = ret;
+		goto error;
+	}
 
 error:
-	if (fp && fclose(fp)) {
-		PERROR("fclose file %s", filepath);
-	}
-	return ret;
+	return fd;
 }
 
 /*
@@ -275,8 +307,8 @@ int utils_mkdir(const char *path, mode_t mode, int uid, int gid)
 	int ret;
 	struct lttng_directory_handle *handle;
 	const struct lttng_credentials creds = {
-		.uid = (uid_t) uid,
-		.gid = (gid_t) gid,
+		.uid = LTTNG_OPTIONAL_INIT_VALUE(uid),
+		.gid = LTTNG_OPTIONAL_INIT_VALUE(gid),
 	};
 
 	handle = lttng_directory_handle_create(NULL);
@@ -304,8 +336,8 @@ int utils_mkdir_recursive(const char *path, mode_t mode, int uid, int gid)
 	int ret;
 	struct lttng_directory_handle *handle;
 	const struct lttng_credentials creds = {
-		.uid = (uid_t) uid,
-		.gid = (gid_t) gid,
+		.uid = LTTNG_OPTIONAL_INIT_VALUE(uid),
+		.gid = LTTNG_OPTIONAL_INIT_VALUE(gid),
 	};
 
 	handle = lttng_directory_handle_create(NULL);
@@ -332,7 +364,7 @@ int utils_stream_file_path(const char *path_name, const char *file_name,
 		char *out_stream_path, size_t stream_path_len)
 {
 	int ret;
-        char count_str[MAX_INT_DEC_LEN(count) + 1] = {};
+	char count_str[MAX_INT_DEC_LEN(count) + 1] = {};
 	const char *path_separator;
 
 	if (path_name && (path_name[0] == '\0' ||
@@ -350,7 +382,7 @@ int utils_stream_file_path(const char *path_name, const char *file_name,
 		assert(ret > 0 && ret < sizeof(count_str));
 	}
 
-        ret = snprintf(out_stream_path, stream_path_len, "%s%s%s%s%s",
+	ret = snprintf(out_stream_path, stream_path_len, "%s%s%s%s%s",
 			path_name, path_separator, file_name, count_str,
 			suffix);
 	if (ret < 0 || ret >= stream_path_len) {
@@ -1030,7 +1062,7 @@ end:
 }
 
 static
-int read_proc_meminfo_field(const char *field, uint64_t *value)
+int read_proc_meminfo_field(const char *field, size_t *value)
 {
 	int ret;
 	FILE *proc_meminfo;
@@ -1048,10 +1080,10 @@ int read_proc_meminfo_field(const char *field, uint64_t *value)
 	 * field.
 	 */
 	while (!feof(proc_meminfo)) {
-		uint64_t value_kb;
+		unsigned long value_kb;
 
 		ret = fscanf(proc_meminfo,
-				"%" MAX_NAME_LEN_SCANF_IS_A_BROKEN_API "s %" SCNu64 " kB\n",
+				"%" MAX_NAME_LEN_SCANF_IS_A_BROKEN_API "s %lu kB\n",
 				name, &value_kb);
 		if (ret == EOF) {
 			/*
@@ -1068,12 +1100,7 @@ int read_proc_meminfo_field(const char *field, uint64_t *value)
 			 * This number is displayed in kilo-bytes. Return the
 			 * number of bytes.
 			 */
-			if (value_kb > UINT64_MAX / 1024) {
-				ERR("Overflow on kb to bytes conversion");
-				break;
-			}
-
-			*value = value_kb * 1024;
+			*value = ((size_t) value_kb) * 1024;
 			ret = 0;
 			goto found;
 		}
@@ -1093,7 +1120,7 @@ fopen_error:
  * a best guess.
  */
 LTTNG_HIDDEN
-int utils_get_memory_available(uint64_t *value)
+int utils_get_memory_available(size_t *value)
 {
 	return read_proc_meminfo_field(PROC_MEMINFO_MEMAVAILABLE_LINE, value);
 }
@@ -1103,7 +1130,7 @@ int utils_get_memory_available(uint64_t *value)
  * the information in `/proc/meminfo`.
  */
 LTTNG_HIDDEN
-int utils_get_memory_total(uint64_t *value)
+int utils_get_memory_total(size_t *value)
 {
 	return read_proc_meminfo_field(PROC_MEMINFO_MEMTOTAL_LINE, value);
 }
@@ -1270,3 +1297,39 @@ end:
 	free(buf);
 	return ret_val;
 }
+
+LTTNG_HIDDEN
+int utils_parse_unsigned_long_long(const char *str,
+		unsigned long long *value)
+{
+	int ret;
+	char *endptr;
+
+	assert(str);
+	assert(value);
+
+	errno = 0;
+	*value = strtoull(str, &endptr, 10);
+
+	/* Conversion failed. Out of range? */
+	if (errno != 0) {
+		/* Don't print an error; allow the caller to log a better error. */
+		DBG("Failed to parse string as unsigned long long number: string = '%s', errno = %d",
+				str, errno);
+		ret = -1;
+		goto end;
+	}
+
+	/* Not the end of the string or empty string. */
+	if (*endptr || endptr == str) {
+		DBG("Failed to parse string as unsigned long long number: string = '%s'",
+				str);
+		ret = -1;
+		goto end;
+	}
+
+	ret = 0;
+
+end:
+	return ret;
+}
